<?php

namespace HCM\FPA\Registro\grrf;

use HCM\FPA\Registro\RegistroGRRF;
use HCM\Util\DateUtil;

class Registro40GRRF extends RegistroGRRF{

    const REGISTRO_40 = '40';
    const SIM = 'S';
    const NAO = 'N';
    const TRABALHADO = '1';
    const INDENIZADO = '2';
    const DISPENSA = '3';
    const PENSAOJUDICIAL = '3';

    private $_pessoa;
    private $_vinculo;
    private $_escalaSemanal;
    private $_tpInscricao;
    private $_nrInscricao;
    private $_tpInscricaoTomador;
    private $_nrInscricaoTomador;
    private $_nrRecolhimento;
    private $_nrCategoria;
    private $_competencia;
    private $_competencia13;
    private $_codMovimentacao;
    private $_codSaque;
    private $_banco;
    private $_agencia;
    private $_contacorrente;
    private $_tpDependente;
    private $_dtHomologacao;
    private $_fgtsmesanterior;
    private $_vrComplAvisoPrevInd;


    public function geraRegistro40($pessoa, $vinculo, $avisoprevioindenizado,
                                   $mesanteriorrescisao, $mesrescisao, $pensaojudicial, $saldofinsrescisorios){

        $this->_pessoa = $pessoa;
        $this->_vinculo = $vinculo;
        $this->_nrRecolhimento = $this->getCodigoRecolhimento();
        $this->_competencia = substr($this->getCompetencia(), 4, 2);
        $this->_competencia13 =  $this->getCompetencia13();
        $this->_vrComplAvisoPrevInd = $avisoprevioindenizado[RegistroGRRF::GRRF_COMPL_AVISO_PREVIO_INDENIZADO];
        $this->_geraRegistro($avisoprevioindenizado, $mesanteriorrescisao, $mesrescisao, $pensaojudicial,
            $saldofinsrescisorios);

    }

    private function _geraRegistro($avisoprevioindenizado, $mesanteriorrescisao, $mesrescisao, $pensaojudicial, $saldofinsrescisorios){

        //bloco 1 ao 3 eh gerado no pai.
        parent::geraRegistro(self::REGISTRO_40, $this->_tpInscricao, $this->_nrInscricao);

        //bloco 4 | 018 | tipo de inscricao do tomador.
        $this->atribuiDado($this->_tpInscricaoTomador, 1, parent::CARACTER_ZERO, parent::PREENCHE_DIREITA);

        //bloco 5 | 019-032 | inscricao do tomador.
        $this->atribuiDado($this->_nrInscricaoTomador, 14, parent::CARACTER_ZERO, parent::PREENCHE_DIREITA);

        //bloco 6 | 033-043 | pis/pasep/ci.
        $identificador = $this->_pessoa->historico->getNrpispaseppes();
        $this->atribuiDado($identificador, 11, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        //bloco 7 | 044-051 | data de admissao.
        $this->_atribuiDataAdmissao($this->_vinculo->getDtadmissaovinc());

        //bloco 8 | 052-053 | categoria do trabalahador.
        //@todo rever campos de categoria na situacao funcional e no vinculo
        $this->atribuiDado($this->_nrCategoria, 2);

        //bloco 9 | 054-123 | nome do trabalhador.
        $nome = $this->_pessoa->historico->getNmpessoa();
        $this->atribuiDado($nome, 70, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        //bloco 10 | 124-130 | numero ctps.
        $this->_atribuiCTPS($this->_pessoa->historico->getNrctpspessoa());

        //bloco 11 | 131-135 | Serie ctps.
        $this->_atribuiSerieCTPS($this->_pessoa->historico->getNrseriectpspes());

        //bloco 12 | 136 | sexo (1 - masculino / 2 - feminino).
        $this->_idSexoPessoa($this->_pessoa->historico->getIdSexoPessoa());

        //bloco 13 | 137-138 | grau de instrucao.
        $nrgrauinstr = (string)$this->_pessoa->historico->getNrGrauInstr();
        $this->atribuiDado($nrgrauinstr, 2, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        
        //bloco 14 | 139-146 | data de nascimento.
        $this->_atribuiDataNascimento($this->_pessoa->historico->getDtnascpessoa());

        //bloco 15 | 147-148 | qt horas trabalhadas por semana.
        $this->atribuiDado($this->_escalaSemanal, 2, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        //bloco 16 | 149-154 | Codigo Brasileiro de Ocupacao.
        $this->_atribuiCbo($this->_vinculo->cbo);

        //bloco 17 | 155-162 | data de opcao.
        $this->_atribuiDataAdmissao($this->_vinculo->getDtadmissaovinc());

        //bloco 18 | 163-164 | codigo de movimentacao.
        $this->_atribuiCodigoMovimentacao();

        //bloco 19 | 165-172 | data de movimentacao.
        $this->_atribuiDataRescisao($this->_vinculo->getDtrescisaovinc());

        //bloco 20 | 173-175 | codigo saque.
        $this->_atribuiCodigoSaque($this->_codSaque);

        //bloco 21 | 176 | aviso previo.
        $avisoprevio = $this->_retornaAvisoPrevio();
        $this->atribuiDado($avisoprevio, 1, parent::CARACTER_ZERO, parent::PREENCHE_DIREITA);

        //bloco 22 | 177-184 | data inicio do aviso previo.
        $this->_atribuiDataAvisoPrevio();

        //bloco 23 | 185 | reposicao de vaga.
        //@todo default 'S' conferir de onde vem esta informacao
        $this->atribuiDado('S', 1, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        //bloco 24 | 186-193 | data da homologacao.
//        $this->atribuiDado(parent::CARACTER_BRANCO, 8, parent::CARACTER_BRANCO, parent::PREENCHE_ESQUERDA);
        $this->_atribuiDataHomologacao();

        //bloco 25 | 194-208 | valor dissidio.
        $this->atribuiDado(parent::CARACTER_ZERO, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
//        $this->_atribuiValorDissidio($remuneracao[RegistroGRRF::GRRF_13SAL_PAGO_MES]);

        //bloco 26 | 209-223 | remuneracao - mes anterior a rescisao.
//        $this->atribuiDado(parent::CARACTER_ZERO, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        $this->_atribuiRemuneracaoMesAnteriorRescisao($mesanteriorrescisao[RegistroGRRF::GRRF_MES_ANTERIOR_RESCISAO]);

        //bloco 27 | 224-238 | remuneracao - mes da rescisao
        $this->_atribuiRemuneracao13($mesrescisao[RegistroGRRF::GRRF_MES_RESCISAO]);

        //bloco 28 | 239-253 | aviso previo indenizado.
        $this->_atribuiAvisoPrevioIndenizado($avisoprevioindenizado[RegistroGRRF::GRRF_AVISO_PREVIO_INDENIZADO]);

        //bloco 29 | 254 | indicativo pensao alimenticia.
        $this->atribuiDado($this->_atribuiPensaoAlimenticia(), 1, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        //bloco 30 | 255-259 | percentual de pensao alimenticia.
        $this->atribuiDado(parent::CARACTER_ZERO, 5, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        //bloco 31 | 260-274 | valor de pensao alimenticia.
        $this->_atribuiPensaoJuridica($pensaojudicial[RegistroGRRF::GRRF_PENSAO_JURIDICA]);

        //bloco 32 | 275-285 | cpf.
        $cpf = $this->_pessoa->historico->getNrcpfpessoa();
        $this->atribuiDado($cpf, 11, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        //bloco 33 | 286-288 | banco da conta do trabalhador.
        //$this->atribuiDado($this->_banco, 3, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        $this->atribuiDado(parent::CARACTER_ZERO, 3, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        //bloco 34 | 289-292 | agencia da conta do trabalhador.
        //$this->atribuiDado($this->_agencia, 4, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        $this->atribuiDado(parent::CARACTER_ZERO, 4, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        //bloco 35 | 293-305 | numero da conta bancaria do trabalhador.
        //$this->atribuiDado($this->_contacorrente, 13, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        $this->atribuiDado(parent::CARACTER_ZERO, 13, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        //bloco 36 | 306-320 | saldo para fins rescisorios.
        $this->_atribuiDadoSaldoRescisorio($saldofinsrescisorios[RegistroGRRF::GRRF_SALDO_FINS_RESCISORIOS]);

        //bloco 37 | 321-359 | preencher com brancos.
        $this->atribuiDado(parent::CARACTER_VAZIO, 39, parent::CARACTER_BRANCO);

        //bloco 38 | 360 | Final de linha. Inserir '*'.
        $this->atribuiDado(parent::CARACTER_VAZIO, 1, parent::CARACTER_ASTERISCO);

    }

    public function setFGTSMesAnterior($fgtsmesanterior)
    {
        $this->_fgtsmesanterior = $fgtsmesanterior;
    }

    public function setDtHomologacao($dtHomologacao)
    {
        $this->_dtHomologacao = $dtHomologacao;
    }

    public function setCodMovimentacao($codMovimentacao)
    {
        $this->_codMovimentacao = $codMovimentacao;
    }

    public function setCodSaque($codsaque)
    {
        $this->_codSaque = $codsaque;
    }

    public function setNrCategoria($nrCategoria)
    {
        $this->_nrCategoria = $nrCategoria;
    }

    public function setNrInscricao($nrInscricao)
    {
        $this->_nrInscricao = $nrInscricao;
    }

    public function setNrInscricaoTomador($nrInscricaoTomador)
    {
        $this->_nrInscricaoTomador = $nrInscricaoTomador;
    }

    public function setTpInscricao($tpInscricao)
    {
        $this->_tpInscricao = $tpInscricao;
    }

    public function setTpInscricaoTomador($tpInscricaoTomador)
    {
        $this->_tpInscricaoTomador = $tpInscricaoTomador;
    }

    public function setEscalaSemanal($escalaSemanal)
    {
        $this->_escalaSemanal = $escalaSemanal;
    }

    public function setBanco($banco)
    {
        $this->_banco = $banco;
    }

    public function setAgencia($agencia)
    {
        $this->_agencia = $agencia;
    }

    public function setContaCorrente($contacorrente)
    {
        $this->_contacorrente = $contacorrente;
    }

    public function setTipoDependente($tpdependente)
    {
        $this->_tpDependente = $tpdependente;
    }
    
    public function setIdAvisoTrabAsInden($vrAvisoTrabIndenGrrf)
    {
        if($vrAvisoTrabIndenGrrf == 1){
            $this->_idAvisoTrabAsInden = true;
        } else {
            $this->_idAvisoTrabAsInden = false;
        }
    }
    
    private function _idSexoPessoa($idSexoPessoa){

        if($idSexoPessoa == 'M'){
            $this->atribuiDado('1', 1, parent::CARACTER_BRANCO, parent::PREENCHE_ESQUERDA);
        }else{
            $this->atribuiDado('2', 1, parent::CARACTER_BRANCO, parent::PREENCHE_ESQUERDA);
        }

    }

    private function _atribuiDadoSaldoRescisorio($saldofinsrescisorios){

        if($this->_dtHomologacao){
            $this->atribuiDado(parent::CARACTER_ZERO, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        }else{
            $this->atribuiDado($saldofinsrescisorios, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        }

    }
    
    private function _atribuiAvisoPrevioIndenizado($avisoprevioindenizado){

        $avisoprevio = array('02');
        if (array_search($this->_retornaAvisoPrevio(), $avisoprevio) !== false) {
            $this->atribuiDado($avisoprevioindenizado, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        }else{
            $this->atribuiDado(parent::CARACTER_ZERO, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        }
    }

    private function _atribuiPensaoJuridica($pensaojuridica){

        $pensaoalimenticia = array('V');
        if (array_search($this->_atribuiPensaoAlimenticia(), $pensaoalimenticia) !== false) {
            $this->atribuiDado($pensaojuridica, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        }else{
            $this->atribuiDado(parent::CARACTER_ZERO, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        }

    }


    private function _atribuiPensaoAlimenticia(){
        if($this->_tpDependente){
            foreach($this->_tpDependente as $dependente){
                if($dependente->getNrtipodepende() == self::PENSAOJUDICIAL){
                    $tpdependente= 'V';
                }else{
                    $tpdependente= 'N';
                }
            }
        }else{
            $tpdependente= 'N';
        }
        return 'N';
    }
    
    private function _retornaAvisoPrevio(){
        if($this->_vinculo->getDtavisoprevio()){
            $dtavisoprevio = DateUtil::getDataDeString($this->_vinculo->getDtavisoprevio());
            $dtrescisao = DateUtil::getDataDeString($this->_vinculo->getDtrescisaovinc());
            
            if($dtavisoprevio < $dtrescisao){
                if($this->_vrComplAvisoPrevInd > 0 && $this->_idAvisoTrabAsInden === true && $this->_codMovimentacao != 'I3'){
                    $avisoprevio = self::INDENIZADO;
                } else {
                    $avisoprevio = self::TRABALHADO;
                }
            }

            if($dtavisoprevio && $dtavisoprevio->format('d/m/Y') === $dtrescisao->format('d/m/Y')){
                $avisoprevio = self::INDENIZADO;
            }
            
            // Força tipo de aviso trabalhado para o tipo de demissão '28 - Rescisão por força maior'
            if($this->_vinculo->getNrtpdemissao() == 28){
                $avisoprevio = self::TRABALHADO;
            }
            
            return $avisoprevio;
        }else{
            return self::DISPENSA;
        }
    }

    private function _atribuiCodigoSaque($codsaque){

        $codmovimentacao = array('H','J','M');
        if (array_search($this->_codMovimentacao, $codmovimentacao) !== false) {
            $this->atribuiDado(parent::CARACTER_BRANCO, 3, parent::CARACTER_BRANCO, parent::PREENCHE_ESQUERDA);
            unset($nrTpDemissao);
        }else{
            $this->atribuiDado($codsaque, 3, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
            unset($nrTpDemissao);
        }
    }
    
    private function _atribuiDataHomologacao(){
        if($this->_dtHomologacao){
            $dataHomologacao = DateUtil::getDataDeString($this->_dtHomologacao);
            $dataHomologacao = $dataHomologacao->format('d/m/Y');
        } else {
            $dataHomologacao = parent::CARACTER_BRANCO;
        }

        $this->atribuiDado($dataHomologacao, 8, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
        unset($dataHomologacao);
    }

    private function _atribuiDataAdmissao($dtAdmissao){

        //obrigatorio para as categorias de trabalhadores 01, 03, 04, 05, 06, 07, 11, 12, 19, 20, 21 e 26.
        $codCategorias = array(
            '01', '03', '04', '05', '06', '07', '11', '12', '19', '20', '21', '26',
        );

        if (array_search($this->_nrCategoria, $codCategorias) !== false) {
            $dataAdmissao = DateUtil::getDataDeString($dtAdmissao);
            $dataAdmissao = $dataAdmissao->format('d/m/Y');
        } else {
            $dataAdmissao = parent::CARACTER_BRANCO;
        }

        $this->atribuiDado($dataAdmissao, 8, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
        unset($dataAdmissao);

    }
    
    private function _atribuiDataNascimento($dtNascimento){
        $codCategorias = array(
            '01', '02', '03', '04', '05', '06', '07', '12', '19', '20', '21', '26',
        );

        if (array_search($this->_nrCategoria, $codCategorias) === false) {
            $dataNascimento = parent::CARACTER_BRANCO;
        } else {
            if (is_null($dtNascimento)) {
                $dataNascimento = parent::CARACTER_BRANCO;
            } else {
                $dataNascimento = DateUtil::getDataDeString($dtNascimento);
                $dataNascimento = $dataNascimento->format('d/m/Y');
            }
        }

        $this->atribuiDado($dataNascimento, 8, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
        unset($dataNascimento);

    }

    private function _atribuiCTPS($nrCTPS){

        $codCategorias = array(
            '01', '02', '03', '04', '06', '07', '26',
        );

        if (array_search($this->_nrCategoria, $codCategorias) === false) {
            $nrCTPS = parent::CARACTER_BRANCO;
        }

        if ((empty($nrCTPS) === true) || ($nrCTPS === parent::CARACTER_BRANCO)) {
            $this->atribuiDado($nrCTPS, 7, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
        } else {
            $this->atribuiDado($nrCTPS, 7, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        }

    }


    private function _atribuiSerieCTPS($serieCTPS){
        /* Obrigatorio para as categorias de trabalhadores 01, 03 , 04, 06, 07 e 26.
         * Opcional para a categoria de trabalhador 02.
         */
        $codCategorias = array(
            '01', '02', '03', '04', '06', '07', '26',
        );

        if (array_search($this->_nrCategoria, $codCategorias) === false) {
            $serieCTPS = parent::CARACTER_BRANCO;
        }

        if ((empty($serieCTPS) === true) || ($serieCTPS === parent::CARACTER_BRANCO)) {
            $this->atribuiDado($serieCTPS, 5, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
        } else {
            $this->atribuiDado($serieCTPS, 5, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        }

    }
    
    private function _atribuiCbo($cbo){
        if (strlen($cbo) > 4){
            $cbo = substr($cbo, 0, 4);
        }
        $this->atribuiDado($cbo, '6', parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

    }
    private function _atribuiCodigoMovimentacao(){

        /* Campo obrigatório, exceto para tipo de remessa igual a 5.
         * Acatarsomente I1,I2, I3, I4 ou L.
         * Não pode acatar código de movimentação I4 para categoria diferente de 06.
         * Não pode acatar código de movimentação I1 para categoria 06.
         * Acatar H,J ou M se tipo de remessa igual a 4.
         * S1, S2, U1, U2 e U3, H,J ou M para implementação futura.
         */

        $codigomovimentacao = array('I1','I2','I3','I4','I5','L');
        if(array_search($this->_codMovimentacao,$codigomovimentacao) !== false){
            $this->atribuiDado($this->_codMovimentacao, '2', parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        }else{
            $this->atribuiDado(parent::CARACTER_ZERO, '2', parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        }
    }
    
    private function _atribuiDataRescisao($dtRescisao){

        //obrigatorio para as categorias de trabalhadores 01, 03, 04, 05, 06, 07, 11, 12, 19, 20, 21 e 26.
        $codCategorias = array(
            '01', '03', '04', '05', '06', '07', '11', '12', '19', '20', '21', '26',
        );

        if (array_search($this->_nrCategoria, $codCategorias) !== false) {
            $dataRecisao = DateUtil::getDataDeString($dtRescisao);
            $dataRecisao = $dataRecisao->format('d/m/Y');
        } else {
            $dataRecisao = parent::CARACTER_BRANCO;
        }

        $this->atribuiDado($dataRecisao, 8, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
        unset($dataRecisao);

    }
    
    private function _atribuiDataAvisoPrevio(){
        $avisoprevio = array('01', '03');
        if (array_search($this->_retornaAvisoPrevio(), $avisoprevio)
            && ($this->_vinculo->getDtavisoprevio())) {

            $dtavisoprevio = DateUtil::getDataDeString($this->_vinculo->getDtavisoprevio());
            $dtrescisao = DateUtil::getDataDeString($this->_vinculo->getDtrescisaovinc());
            $dtadmissao = DateUtil::getDataDeString($this->_vinculo->getDtadmissaovinc());

            if($dtavisoprevio > $dtadmissao || $dtavisoprevio->format('d/m/Y') === $dtadmissao->format('d/m/Y') && $dtavisoprevio < $dtrescisao || $dtavisoprevio->format('d/m/Y') === $dtrescisao->format('d/m/Y')){
                $dataFormatada = DateUtil::getDataDeString($dtavisoprevio);
                $dataFormatada = $dataFormatada->format('d/m/Y');
                $this->atribuiDado($dataFormatada, 8, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
                unset($dataFormatada);
            }

        }else{
            $this->atribuiDado(parent::CARACTER_BRANCO, 8, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
            unset($dataFormatada);
        }

    }

    private function _atribuiRemuneracaoMesAnteriorRescisao($mesanteriorrescisao){

        if(!$this->_dtHomologacao && $this->_fgtsmesanterior === 'S'){

            $dtadmissao = DateUtil::getDataDeString($this->_vinculo->getDtadmissaovinc());
            $dtadmissao = $dtadmissao->format('m/Y');
            $dtmovimentacao = DateUtil::getDataDeString($this->_vinculo->getDtrescisaovinc());
            $dtmovimentacao = $dtmovimentacao->format('m/Y');

            if($dtadmissao != $dtmovimentacao ){
                $resultado = $mesanteriorrescisao;
            }else{
                $resultado = parent::CARACTER_ZERO;
            }

        }else {
            $resultado = parent::CARACTER_ZERO;
        }

        $this->atribuiDado($resultado, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

    }

    private function _atribuiRemuneracao13($remuneracao13){
        
        $codCategorias = array(
            '01', '02', '03', '04', '06', '07', '12', '19', '20', '21', '26',
        );

        if ((array_search($this->_nrCategoria, $codCategorias) === false)
            ||($this->_competencia13 === true)
        ) {
            $remuneracao13 = parent::CARACTER_ZERO;
        }

        $this->atribuiDado($remuneracao13, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
    }
}