<?php

namespace HCM\PTO\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\DateUtil;

class BancoHorasController extends Crud {

    protected $parameterBagColumns = array('NRORG');
    protected $dataSourceName = "getBancoHoras";
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters   = $hcmParameters;
        parent::__construct($dataSourceManager);
    }

    public function getBancoHoras (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $this->parameterBagColumns = array('NRORG', 'IDSITUFREQS');
            $filterData = $request->getFilterCriteria();
            $conditions = array(); 
            foreach ($filterData->getconditions() as $key => $condition) {
                $conditions[$key]['columnName'] = $condition['columnName'];
                $conditions[$key]['operator'] = $condition['operator'];
                if($condition['columnName'] === 'IDSITUFREQS'){
                    $conditions[$key]['value'] = implode(',', $condition['value']);
                }else{
                    $conditions[$key]['value'] = $condition['value'];
                }
            }
            $filtro = new filterCriteria(
                $this->dataSourceName,
                $conditions,
                $filterData->getPage(),
                $filterData->getPageSize()    
            );
            $newFilterData = $this->populateParameterBag($filtro);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $rows = array();
            foreach ($dataSet->getRows() as $row) {
                $row['IDSITUFREQS'] = explode(',', $row['IDSITUFREQS']);
                foreach($row['IDSITUFREQS'] as $chave => $value){
                    $row['IDSITUFREQS'][$chave] = intval($value);
                }
                array_push($rows, $row);
            }
            $response->addDataSet(new DataSet($this->dataSourceName, $rows));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

     public function getDadosPeriodo (DTO\Request\Filter $request, DTO\Response $response) {
        try{
           $filterCriteria = $request->getFilterCriteria();
            $this->dataSourceName = "getBancoHorasPeriodo";
            parent::find($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setBancoHoras (DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            $this->dataSourceName = "getBancoHoras";
            foreach($request->getDataSet()->getRows() as &$row) {
                if(array_key_exists('IDSITUFREQS', $row)){
                    $row['IDSITUFREQS'] = implode(',', $row['IDSITUFREQS']);
                }
                if(array_key_exists('NRBANCOHORAS', $row)){
                    if(empty($row['NRBANCOHORAS'])){
                        $row['NRBANCOHORAS'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'PTO_BANCOHORAS', 1, 12, true);
                    }
                } else {
                    $row['NRBANCOHORAS'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'PTO_BANCOHORAS', 1, 12, true);
                }
                if(array_key_exists('IDATIVO', $row)){
                    if($row['IDATIVO'] === ''){
                    
                    $row['IDATIVO'] = 'S';
                    }
                } else {
                    $row['IDATIVO'] = 'S';
                }
                
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function setBancoHorasPeriodo (DTO\Request\DataSet $request, DTO\Response $response) {
        
        try{
            $this->dataSourceName = "getBancoHorasPeriodo";
            foreach($request->getDataSet()->getRows() as &$row) {
                if(array_key_exists('NRBANCOHORASPERIODO', $row)){
                    if(empty($row['NRBANCOHORASPERIODO'])){
                        $row['NRBANCOHORASPERIODO'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'PTO_BANCOHORASPERIODO', 1, 12, true);
                    }
                } else {
                    $row['NRBANCOHORASPERIODO'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'PTO_BANCOHORASPERIODO', 1, 12, true);
                }
                if(array_key_exists('IDATIVO', $row)){
                    if($row['IDATIVO'] === ''){
                    
                    $row['IDATIVO'] = 'S';
                    }
                } else {
                    $row['IDATIVO'] = 'S';
                }
                
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deletePeriodo(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = "getBancoHorasPeriodo";
            parent::delete($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
     public function delete(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = "getBancoHoras";
            parent::delete($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
}