<?php
namespace Planejamento\Repositories;

use Geral\Entities\Produto;

class Recepadr extends \Doctrine\ORM\EntityRepository
{

    public static function verificaProdInativoFilial($nrorg, $filial, $prato, $filAux, $maxNivelProd)
    {
        $query = "SELECT P.NMPRODUTO, P.CDARVPROD, P.IDPRODATIVPR, NVL(PF.IDPRODATIV, 'A') AS IDPRODATIV
                  FROM PRODUTO P
                  RIGHT JOIN PRODFILI PF ON PF.CDFILIAL = :P_CDFILIAL
                                       AND P.CDPRODUTO = PF.CDPRODUTO
                                       AND P.NRORG = PF.NRORG
                  RIGHT JOIN RECEPADR RP ON RP.CDPRATO      = :P_CDPRATO
                                       AND RP.CDFILAUXPRAT = :P_CDFILAUXPRAT
                                       AND RP.CDPRODUTO    = P.CDPRODUTO
                                       AND RP.NRORG = P.NRORG
                 WHERE P.CDNVPRODUTO = :P_CDNVPRODUTO
                   AND P.NRORG = :P_NRORG";

        $param = array(
            "P_CDNVPRODUTO" => $maxNivelProd,
            "P_CDFILIAL" => $filial,
            "P_CDPRATO" => $prato,
            "P_CDFILAUXPRAT" => $filAux,
            "P_NRORG" => $nrorg
        );

        try {
            $conn = \Setforms2\Runtime\Connection::getInstance();
            $result = $conn->fetchAll($query, $param);
            return $result;
        } catch (\Exception $e) {
            throw $e;
        }

    }

    public static function atualizaProdFatorCorrecaoLiquida($produto, $quantidade, $prato, $nrorg, $filAux, $quanValorNutri, $VRPEFATCORPR)
    {

        $query = "UPDATE RECEPADR
                  SET VRPEFATCORPR = :VRPEFATCORPR,
                      QTPRODPRAT = :QTPRODPRAT,
                      QTCALCNUTRPR = :QTCALCNUTRPR
                  WHERE CDFILAUXPRAT = :OLD_CDFILAUXPRAT
                        AND CDPRATO = :OLD_CDPRATO
                        AND CDPRODUTO = :OLD_CDPRODUTO
                        AND NRORG = :OLD_NRORG";

        $param = array(
            "VRPEFATCORPR" => $VRPEFATCORPR,
            "QTCALCNUTRPR" => $quanValorNutri,
            "OLD_CDPRODUTO" => $produto,
            "QTPRODPRAT" => $quantidade,
            "OLD_CDPRATO" => $prato,
            "OLD_CDFILAUXPRAT" => $filAux,
            "OLD_NRORG" => $nrorg
        );

        try {
            $conn = \Setforms2\Runtime\Connection::getInstance();
            $conn->execute($query, $param);
            return true;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function atualizaProdFatorCorrecaoBruta($produto, $quantidade, $prato, $nrorg, $filAux, $quanValorNutri, $VRPEFATCORPR)
    {

        $query = "UPDATE RECEPADR
                  SET VRPEFATCORPR = :VRPEFATCORPR,
                      QTBRUTAPRUNR = :QTBRUTAPRUNR,
                      QTCALCNUTRPR = :QTCALCNUTRPR
                  WHERE CDFILAUXPRAT = :OLD_CDFILAUXPRAT
                        AND CDPRATO = :OLD_CDPRATO
                        AND CDPRODUTO = :OLD_CDPRODUTO
                        AND NRORG = :OLD_NRORG";

        $param = array(
            "VRPEFATCORPR" => $VRPEFATCORPR,
            "QTCALCNUTRPR" => $quanValorNutri,
            "OLD_CDPRODUTO" => $produto,
            "QTBRUTAPRUNR" => $quantidade,
            "OLD_CDPRATO" => $prato,
            "OLD_CDFILAUXPRAT" => $filAux,
            "OLD_NRORG" => $nrorg
        );

        try {
            $conn = \Setforms2\Runtime\Connection::getInstance();
            $conn->execute($query, $param);
            return true;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function deletaReceita($cdprato, $cdfilauxprat, $nrorg ) {
        $query = "DELETE FROM RECEPADR
                  WHERE
                    CDPRATO      = :CDPRATDEST   AND
                    CDFILAUXPRAT = :CDFILAUXDEST AND
                    NRORG        = :NRORG";

        $param = array(
            "CDPRATDEST" => $cdprato,
            "CDFILAUXDEST" => $cdfilauxprat,
            "NRORG" => $nrorg
        );

        try {
            $conn = \Setforms2\Runtime\Connection::getInstance();
            $conn->execute($query, $param);
            return true;
        } catch (\Exception $e) {
            throw $e;
        }

    }

    public static function sobrepoeReceitaPadrao($proporcao, $cdprato, $cdfilauxprat, $cdfilcentpme = null
                                                    , $cdtipocard = null, $receitaorig, $cdfiliorig, $cdservorig, $nrorg) {
        $query = "INSERT INTO RECEPADR
                    ( CDPRATO,
                      CDFILAUXPRAT,
                      CDPRODUTO,
                      QTPRODPRAT,
                      SGUNIDADE,
                      VRPEFATCORPR,
                      VRPEFATCOCPR,
                      HRTEMPPREPPR,
                      QTUNIDESTPR,
                      QTBRUTAPR,
                      CDAPRERECE,
                      QTBRUTAPRUNR,
                      VRPEFATNUTPR,
                      QTCALCNUTRPR,
                      IDQTDEFIXPR)


                    (SELECT CDPRATO, CDFILAUXPRAT, CDPRODUTO, (QTPRODPRAT * :PROPORCAO)     AS QTPRODUTO,
                      SGUNIDADE, VRPEFATCORPR    AS VRPEFATCOR, VRPEFATCOCPR    AS VRPEFATCOC, HRTEMPPREPPR
                      AS HRTEMPPREP, (QTUNIDESTPR  * :PROPORCAO)   AS QTDERECE, (QTBRUTAPR    * :PROPORCAO)
                      AS QTBRUTA, CDAPRERECE, (QTBRUTAPRUNR * :PROPORCAO)  AS    QTBRUTAUNR, VRPEFATNUTPR
                      AS  VRPEFATNUT, (QTCALCNUTRPR * :PROPORCAO)  AS   QTCALCNUTR, IDQTDEFIXPR
                     FROM
                      RECEPADR
                     WHERE (CDPRATO       = :CDPRATORIG)
                       AND (CDFILAUXPRAT  = :CDFILAUXORIG)
                       AND (:IDTIPORECE = 'P')

                    UNION ALL

                     SELECT CDPRATO, CDFILAUXPRAT, CDPRODUTO, (QTPRODPRATTP * :PROPORCAO) AS QTPRODUTO, SGUNIDADE,
                        VRPEFATCORPRTP AS VRPEFATCOR, VRPEFATCOCPRTP AS VRPEFATCOC, HRTEMPPREPPRTP AS HRTEMPPREP,
                        (QTUNIDESTPRTP * :PROPORCAO) AS QTDERECE, (QTBRUTAPRTP * :PROPORCAO) AS QTBRUTA, CDAPRERECE,
                        (QTBRUTAPRUNRTP * :PROPORCAO) AS QTBRUTAUNR, VRPEFATNUTPRTP AS VRPEFATNUT, (QTCALCNUTRPRTP * :PROPORCAO)
                        AS QTCALCNUTR, IDQTDEFIXPRTP
                      FROM
                        RECETPCARD
                      WHERE (CDPRATO       = :CDPRATORIG)
                        AND (CDFILAUXPRAT  = :CDFILAUXORIG)
                        AND (CDFILCENTPME  = :CDFILCENTPME)
                        AND (CDTIPOCARD    = :CDTIPOCARD)
                        AND (:IDTIPORECE = 'T')

                     UNION ALL

                     SELECT CDPRATO, CDFILAUXPRAT, CDPRODUTO, (QTPRODRECLOC * :P_PROPORCAO) AS QTPRODUTO,
                       SGUNIDADE, VRPEFATCORLO AS VRPEFATCOR, VRPEFATCOCLO AS VRPEFATCOC, HRTEMPPREPLO AS HRTEMPPREP,
                       (QTUNIDESTLO * :P_PROPORCAO)  AS QTDERECE, (QTBRUTALO * :P_PROPORCAO)   AS QTBRUTA,
                       CDAPRERECE, (QTBRUTALOUNR * :P_PROPORCAO)   AS    QTBRUTAUNR, VRPEFATNUTLO   AS  VRPEFATNUT,
                       (QTCALCNUTRLO * :P_PROPORCAO)  AS   QTCALCNUTR, IDQTDEFIXLO
                     FROM
                        RECEFILI
                     WHERE  (CDPRATO       = :P_CDPRATORIG)
                        AND (CDFILAUXPRAT  = :P_CDFILAUXORIG)
                        AND (CDFILIAL      = :P_CDFILIORIG)
                        AND (:IDTIPORECE = 'F')

                     UNION ALL

                    SELECT CDPRATO, CDFILAUXPRAT, CDPRODUTO, (QTPRODRECSER * :P_PROPORCAO) AS QTPRODUTO,
                        SGUNIDADE, VRPEFATCORSE AS VRPEFATCOR, VRPEFATCOCSE AS VRPEFATCOC,HRTEMPPREPSE AS HRTEMPPREP,
                        (QTUNIDESTSE * :P_PROPORCAO)  AS QTDERECE, (QTBRUTASE * :P_PROPORCAO)   AS QTBRUTA,
                        CDAPRERECE, (QTBRUTASEUNR * :P_PROPORCAO)   AS    QTBRUTAUNR, VRPEFATNUTSE   AS  VRPEFATNUT,
                        (QTCALCNUTRSE * :P_PROPORCAO)  AS   QTCALCNUTR, IDQTDEFIXSE
                     FROM
                        RECESERV
                     WHERE  (CDPRATO       = :P_CDPRATORIG)
                        AND (CDFILAUXPRAT  = :P_CDFILAUXORIG)
                        AND (CDFILIAL      = :P_CDFILIORIG)
                        AND (CDSERVFILI    = :P_CDSERVORIG)
                        AND (:IDTIPORECE = 'S')";

        $param = array(
            "PROPORCAO"    => $proporcao,
            "CDPRATORIG"   => $cdprato,
            "CDFILAUXORIG" => $cdfilauxprat,
            "IDTIPORECE"   => $receitaorig,
            "CDFILCENTPME" => $cdfilcentpme,
            "CDTIPOCARD"   => $cdtipocard,
            "CDFILIORIG"   => $cdfiliorig,
            "CDSERVORIG"   => $cdservorig,
            "NRORG"        => $nrorg
        );

        try {
            $conn = \Setforms2\Runtime\Connection::getInstance();
            $conn->execute($query, $param);
            return true;
        } catch (\Exception $e) {
            throw $e;
        }

    }

    public static function acrescentaReceitaPadrao($proporcao, $cdprato, $cdfilauxprat, $cdfilcentpme = null,
                                                   $cdtipocard = null, $receitaorig, $cdfiliorig, $cdservorig, $nrorg) {

        $query = "INSERT INTO RECEPADR
                    (CDPRATO,
                    CDFILAUXPRAT,
                    CDPRODUTO,
                    QTPRODPRAT,
                    SGUNIDADE,
                    VRPEFATCORPR,
                    VRPEFATCOCPR,
                    HRTEMPPREPPR,
                    QTUNIDESTPR,
                    QTBRUTAPR,
                    CDAPRERECE,
                    QTBRUTAPRUNR,
                    VRPEFATNUTPR,
                    QTCALCNUTRPR,
                    IDQTDEFIXPR)

                 (SELECT
                    CDPRATO,
                    CDFILAUXPRAT,
                    CDPRODUTO,
                    (QTPRODPRAT * :PROPORCAO)  AS QTPRODUTO,
                    SGUNIDADE,
                    VRPEFATCORPR AS VRPEFATCOR,
                    VRPEFATCOCPR AS VRPEFATCOC,
                    HRTEMPPREPPR AS HRTEMPPREP,
                    (QTUNIDESTPR * :PROPORCAO) AS QTDERECE,
                    (QTBRUTAPR * :PROPORCAO)   AS QTBRUTA,
                    CDAPRERECE,
                    (QTBRUTAPRUNR * :PROPORCAO)  AS    QTBRUTAUNR,
                    VRPEFATNUTPR   AS  VRPEFATNUT,
                    (QTCALCNUTRPR * :PROPORCAO) AS   QTCALCNUTR,
                    IDQTDEFIXPR
                 FROM
                    RECEPADR
                 WHERE
                    (CDPRATO       = :CDPRATO)   AND
                    (CDFILAUXPRAT  = :CDFILAUX) AND
                    (:IDTIPORECE = 'P')             AND
                 (CDPRODUTO NOT IN ( SELECT CDPRODUTO
                                     FROM RECEPADR
                                     WHERE
                                      CDPRATO      = :CDPRATO AND
                                      CDFILAUXPRAT = :CDFILAUX ))

                  UNION ALL

                  SELECT
                    CDPRATO,
                    CDFILAUXPRAT,
                    CDPRODUTO,
                    (QTPRODPRATTP * :PROPORCAO)  AS QTPRODUTO,
                    SGUNIDADE,
                    VRPEFATCORPRTP AS VRPEFATCOR,
                    VRPEFATCOCPRTP AS VRPEFATCOC,
                    HRTEMPPREPPRTP AS HRTEMPPREP,
                    (QTUNIDESTPRTP * :PROPORCAO) AS QTDERECE,
                    (QTBRUTAPRTP * :PROPORCAO)   AS QTBRUTA,
                    CDAPRERECE,
                    (QTBRUTAPRUNRTP * :PROPORCAO)  AS    QTBRUTAUNR,
                    VRPEFATNUTPRTP   AS  VRPEFATNUT,
                    (QTCALCNUTRPRTP * :PROPORCAO)  AS   QTCALCNUTR,
                    IDQTDEFIXPRTP
                 FROM
                    RECETPCARD
                 WHERE
                    (CDPRATO       = :CDPRATO)   AND
                    (CDFILAUXPRAT  = :CDFILAUX) AND
                    (CDFILCENTPME  = :CDFILCENTPME) AND
                    (CDTIPOCARD    = :CDTIPOCARD) AND
                    (:IDTIPORECE = 'T')             AND
                    (CDPRODUTO NOT IN ( SELECT CDPRODUTO
                                        FROM RECEPADR
                                        WHERE
                                          CDPRATO      = :CDPRATO AND
                                          CDFILAUXPRAT = :CDFILAUX ))


                 UNION ALL

                 SELECT
                    CDPRATO,
                    CDFILAUXPRAT,
                    CDPRODUTO,
                    (QTPRODRECLOC * :PROPORCAO) AS QTPRODUTO,
                    SGUNIDADE,
                    VRPEFATCORLO AS VRPEFATCOR,
                    VRPEFATCOCLO AS VRPEFATCOC,
                    HRTEMPPREPLO AS HRTEMPPREP,
                    (QTUNIDESTLO * :PROPORCAO) AS QTDERECE,
                    (QTBRUTALO * :PROPORCAO)   AS QTBRUTA,
                    CDAPRERECE,
                    (QTBRUTALOUNR * :PROPORCAO)  AS    QTBRUTAUNR,
                    VRPEFATNUTLO   AS  VRPEFATNUT,
                    (QTCALCNUTRLO * :PROPORCAO) AS   QTCALCNUTR,
                    IDQTDEFIXLO
                 FROM
                    RECEFILI
                 WHERE
                    (CDPRATO       = :CDPRATO)     AND
                    (CDFILAUXPRAT  = :CDFILAUX)    AND
                    (CDFILIAL      = :CDFILIORIG)  AND
                    (:IDTIPORECE = 'F')            AND
                    (CDPRODUTO NOT IN ( SELECT CDPRODUTO
                                        FROM RECEPADR
                                        WHERE
                                          CDPRATO      = :CDPRATO AND
                                          CDFILAUXPRAT = :CDFILAUX ))

                 UNION ALL

                 SELECT
                    CDPRATO,
                    CDFILAUXPRAT,
                    CDPRODUTO,
                    (QTPRODRECSER * :PROPORCAO) AS QTPRODUTO,
                    SGUNIDADE,
                    VRPEFATCORSE AS VRPEFATCOR,
                    VRPEFATCOCSE AS VRPEFATCOC,
                    HRTEMPPREPSE AS HRTEMPPREP,
                    (QTUNIDESTSE * :PROPORCAO) AS QTDERECE,
                    (QTBRUTASE * :PROPORCAO)   AS QTBRUTA,
                    CDAPRERECE,
                    (QTBRUTASEUNR * :PROPORCAO)  AS    QTBRUTAUNR,
                    VRPEFATNUTSE   AS  VRPEFATNUT,
                    (QTCALCNUTRSE * :PROPORCAO) AS   QTCALCNUTR,
                    IDQTDEFIXSE
                 FROM
                    RECESERV
                 WHERE
                    (CDPRATO       = :CDPRATO)      AND
                    (CDFILAUXPRAT  = :CDFILAUX)     AND
                    (CDFILIAL      = :CDFILIORIG)   AND
                    (CDSERVFILI    = :CDSERVORIG)   AND
                    (:IDTIPORECE = 'S')             AND
                    (CDPRODUTO NOT IN ( SELECT CDPRODUTO
                                        FROM RECEPADR
                                        WHERE
                                          CDPRATO      = :CDPRATO AND
                                          CDFILAUXPRAT = :CDFILAUX ))

                 )";

        $param = array(
            "PROPORCAO"    => $proporcao,
            "CDPRATO"      => $cdprato,
            "CDFILAUX"     => $cdfilauxprat,
            "IDTIPORECE"   => $receitaorig,
            "CDFILCENTPME" => $cdfilcentpme,
            "CDTIPOCARD"   => $cdtipocard,
            "CDFILIORIG"   => $cdfiliorig,
            "CDSERVORIG"   => $cdservorig,
            "NRORG"        => $nrorg
        );
//:P_PROPORCAO(FLOAT,IN)=1
//:P_CDPRATO(VARCHAR[10],IN)='0101010000'
//:P_CDFILAUX(VARCHAR[1],IN)='P'
//:P_IDTIPORECE(VARCHAR[1],IN)='F'
//:P_CDFILCENTPME(VARCHAR[0],IN)=<NULL>
//:P_CDTIPOCARD(VARCHAR[0],IN)=<NULL>
//:P_CDFILIORIG(VARCHAR[4],IN)='9999'
//:P_CDSERVORIG(VARCHAR[0],IN)=<NULL>
        try {
            $conn = \Setforms2\Runtime\Connection::getInstance();
            $conn->execute($query, $param);
            return true;
        } catch (\Exception $e) {
            throw $e;
        }


    }

}